@@ -11,22 +11,23 @@ $sipc->connect(
    Proto      => 'tcp',
    Listen     => 10,
    Reuse      => 1,
-) or die $sipc->errstr($@);
+) or die $sipc->errstr;
 
-$sip->debug(1);
-$sipc->sock->timeout(10);
+$sipc->debug(1);
 
 while ( 1 ) {
-   while (my $client = $sipc->accept()) {
+   my $client;
+   while ( $client = $sipc->accept(10) ) {
       print "connect from client: ", $client->sock->peerhost, "\n";
-      my $request = $client->read_raw or die $client->errstr($!);
+      my $request = $client->read_raw or die $client->errstr;
       next unless $request;
       chomp($request);
       warn "client says: $request\n";
-      $client->send({ foo => 'is foo', bar => 'is bar', baz => 'is baz'}) or die $client->errstr($!);
-      $client->disconnect or die $client->errstr($!);
+      $client->send({ foo => 'is foo', bar => 'is bar', baz => 'is baz'}) or die $client->errstr;
+      $client->disconnect or die $client->errstr;
    }
+   die $sipc->errstr unless defined $client;
    warn "server runs on a timeout, re-listen on socket\n";
 }
 
-$sipc->disconnect or die $sipc->errstr($!);
+$sipc->disconnect or die $sipc->errstr;
