@@ -16,23 +16,23 @@ $sipc->connect(
    SSL_cert_file   => '../certs/servercert.pem',
    SSL_key_file    => '../certs/serverkey.pem',
    SSL_passwd_cb   => sub {return "megaraptor"},
-) or die $sipc->errstr($sipc->sock->errstr);
+) or die $sipc->errstr;
 
-warn "server initialized\n";
+$sipc->debug(1);
 
-$sipc->sock->timeout(10);
-
-while ( 1 ) {
-   while (my $client = $sipc->accept()) {
+while ( 1 ) { 
+   my $client;
+   while ( $client = $sipc->accept(10) ) { 
       print "connect from client: ", $client->sock->peerhost, "\n";
-      my $request = $client->read_raw or die $client->errstr($client->sock->errstr);
+      my $request = $client->read_raw or die $client->errstr;
       next unless $request;
       chomp($request);
       warn "client says: $request\n";
-      $client->send({ foo => 'is foo', bar => 'is bar', baz => 'is baz'}) or die $client->errstr($client->sock->errstr);
-      $client->disconnect or die $client->errstr($!);
-   }
+      $client->send({ foo => 'is foo', bar => 'is bar', baz => 'is baz'}) or die $client->errstr;
+      $client->disconnect or die $client->errstr;
+   }   
+   die $sipc->errstr unless defined $client;
    warn "server runs on a timeout, re-listen on socket\n";
 }
 
-$sipc->disconnect or die $sipc->errstr($!);
+$sipc->disconnect or die $sipc->errstr;
